"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ComponentEngine = void 0;

var _dom = require("./dom");

var _isBrowserEnv = require("./is-browser-env");

var isString = function isString(value) {
  return typeof value === "string";
};

var isFunction = function isFunction(value) {
  return typeof value === "function";
};

var ComponentEngine = {
  start: function start() {
    var metadata = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    if (!_isBrowserEnv.isBrowserEnv) return;
    var id = metadata.id,
        attribute = metadata.attribute,
        thisArg = metadata.thisArg;

    if (id && isString(id)) {
      var instance = document.querySelector("[".concat(attribute, "='").concat(id, "']"));
      if (!instance) return;
      var validComponent = [instance].filter(thisArg._validate)[0];
      if (!validComponent) return;

      thisArg._components.push(validComponent);
    } else if (!id && !thisArg._components.length) {
      var instances = (0, _dom.queryAll)("[".concat(attribute, "]"));
      if (!instances.length) return;
      var validComponents = instances.filter(thisArg._validate);
      thisArg._components = thisArg._components.concat(validComponents);
    } else {}
  },
  stop: function stop() {
    var metadata = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    if (!_isBrowserEnv.isBrowserEnv) return;
    var id = metadata.id,
        attribute = metadata.attribute,
        thisArg = metadata.thisArg,
        activeNodeKey = metadata.activeNodeKey,
        cancelActiveFn = metadata.cancelActiveFn;

    if (id && isString(id)) {
      var targetIndex;

      var instance = thisArg._components.filter(function (activeInstance, index) {
        if (activeInstance.getAttribute(attribute) !== id) return false;
        targetIndex = index;
        return true;
      })[0];

      if (!instance) return;
      if (thisArg[activeNodeKey] && instance === thisArg[activeNodeKey] && isFunction(cancelActiveFn)) thisArg[cancelActiveFn]();

      thisArg._teardown(instance);

      thisArg._components.splice(targetIndex, 1);
    } else if (!id && thisArg._components.length) {
      if (thisArg[activeNodeKey] && isFunction(cancelActiveFn)) thisArg[cancelActiveFn]();

      thisArg._components.forEach(thisArg._teardown);

      thisArg._components = [];
    }
  }
};
exports.ComponentEngine = ComponentEngine;
//# sourceMappingURL=component-engine.js.map