"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createFocusRing = void 0;

var _constants = require("./constants");

var _isBrowserEnv = require("./is-browser-env");

var createFocusRing = function createFocusRing() {
  if (!_isBrowserEnv.isBrowserEnv) return;
  var listeningForKeydown;

  var listenForKeyboard = function listenForKeyboard() {
    document.body.classList.add(_constants.Selectors.KEYBOARD_CLASS);
    document.removeEventListener(_constants.Events.KEYDOWN, listenForKeyboard);
    document.addEventListener(_constants.Events.CLICK, listenForClick);
    listeningForKeydown = false;
  };

  var listenForClick = function listenForClick() {
    document.body.classList.remove(_constants.Selectors.KEYBOARD_CLASS);
    document.removeEventListener(_constants.Events.CLICK, listenForClick);
    document.addEventListener(_constants.Events.KEYDOWN, listenForKeyboard);
    listeningForKeydown = true;
  };

  return {
    start: function start() {
      document.addEventListener(_constants.Events.KEYDOWN, listenForKeyboard);
    },
    stop: function stop() {
      if (listeningForKeydown) {
        document.removeEventListener(_constants.Events.KEYDOWN, listenForKeyboard);
      } else {
        document.body.classList.remove(_constants.Selectors.KEYBOARD_CLASS);
        document.removeEventListener(_constants.Events.CLICK, listenForClick);
      }
    }
  };
};

exports.createFocusRing = createFocusRing;
//# sourceMappingURL=create-focus-ring.js.map