"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createFocusTrap = void 0;

var _isBrowserEnv = require("./is-browser-env");

var _constants = require("./constants");

var _log = require("./log");

var _dom = require("./dom");

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

var createFocusTrap = function createFocusTrap(selectorString) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  if (!_isBrowserEnv.isBrowserEnv) return;
  var useArrows = options.useArrows,
      children = options.children,
      _options$matchers = options.matchers,
      matchers = _options$matchers === void 0 ? _constants.Selectors.FOCUSABLE_TAGS : _options$matchers;

  if (!selectorString && !children.length) {
    (0, _log.log)(_constants.Messages.NO_SELECTOR_STRING_OR_CHILDREN_ERROR);
    return;
  }

  if (!Array.isArray(matchers)) {
    (0, _log.log)(_constants.Messages.OPTION_MATCHERS_DATA_TYPE_ERROR);
    return;
  } else if (matchers.length) {
    var hasBadMatcher = false;
    matchers.forEach(function (matcher) {
      var type = _typeof(matcher);

      if (type !== "string") {
        (0, _log.log)(_constants.Messages.INCORRECT_MATCHER_TYPE_ERROR(type));
        hasBadMatcher = true;
      }
    });
    if (hasBadMatcher) return;
  } else if (!matchers.length) {
    (0, _log.log)(_constants.Messages.NO_MATCHER_LENGTH_ERROR);
    return;
  }

  var focusableChildren = children && children.length ? children : (0, _dom.getFocusableElements)(selectorString, matchers);
  var focusableFirstChild = focusableChildren[0];
  var focusableLastChild = focusableChildren[focusableChildren.length - 1];

  var handleFocusTrapWithTab = function handleFocusTrapWithTab(event) {
    var containerElement = document.querySelector(selectorString);
    var containerActive = document.activeElement === containerElement;
    var firstActive = document.activeElement === focusableFirstChild;
    var lastActive = document.activeElement === focusableLastChild;
    if (!containerActive && !firstActive && !lastActive) return;
    var tabKey = event.which === _constants.KeyCodes.TAB;
    var shiftKey = event.which === _constants.KeyCodes.SHIFT || event.shiftKey;
    var hasShift = shiftKey && tabKey;
    var noShift = !shiftKey && tabKey;

    if (hasShift && (firstActive || containerActive)) {
      event.preventDefault();
      focusableLastChild.focus();
    } else if (noShift && lastActive) {
      event.preventDefault();
      focusableFirstChild.focus();
    }
  };

  var focusNextChild = function focusNextChild() {
    for (var i = 0; i < focusableChildren.length; i++) {
      if (focusableChildren[i] === document.activeElement) {
        focusableChildren[i + 1].focus();
        break;
      }
    }
  };

  var focusLastChild = function focusLastChild() {
    for (var i = 0; i < focusableChildren.length; i++) {
      if (focusableChildren[i] === document.activeElement) {
        focusableChildren[i - 1].focus();
        break;
      }
    }
  };

  var handleFocusTrapWithArrows = function handleFocusTrapWithArrows(event) {
    var firstActive = document.activeElement === focusableFirstChild;
    var lastActive = document.activeElement === focusableLastChild;
    var arrowUp = event.which === _constants.KeyCodes.ARROW_UP;
    var arrowDown = event.which === _constants.KeyCodes.ARROW_DOWN;

    if (arrowUp || arrowDown) {
      event.preventDefault();

      if (firstActive && arrowUp) {
        focusableLastChild.focus();
      } else if (lastActive && arrowDown) {
        focusableFirstChild.focus();
      } else if (arrowDown) {
        focusNextChild();
      } else if (arrowUp) {
        focusLastChild();
      }
    }
  };

  return {
    start: function start() {
      if (useArrows) {
        document.addEventListener(_constants.Events.KEYDOWN, handleFocusTrapWithArrows);
      } else {
        document.addEventListener(_constants.Events.KEYDOWN, handleFocusTrapWithTab);
      }
    },
    stop: function stop() {
      if (useArrows) {
        document.removeEventListener(_constants.Events.KEYDOWN, handleFocusTrapWithArrows);
      } else {
        document.removeEventListener(_constants.Events.KEYDOWN, handleFocusTrapWithTab);
      }
    }
  };
};

exports.createFocusTrap = createFocusTrap;
//# sourceMappingURL=create-focus-trap.js.map