"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _helpers = require("../helpers");

var _constants = require("./constants");

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var COMPONENT_ROLE = "tooltip";

var Tooltip = function () {
  function Tooltip() {
    _classCallCheck(this, Tooltip);

    this._handleEvent = this._handleEvent.bind(this);
    this._handleClose = this._handleClose.bind(this);
    this._handleEscapeKeyPress = this._handleEscapeKeyPress.bind(this);
    this._validate = this._validate.bind(this);
    this._teardown = this._teardown.bind(this);
    this._components = [];
    this._activeTrigger = null;
    this._activeTooltipBox = null;
  }

  _createClass(Tooltip, [{
    key: "start",
    value: function start(id) {
      _helpers.ComponentEngine.start({
        id: id,
        attribute: _constants.Selectors.DATA_TOOLTIP,
        thisArg: this
      });
    }
  }, {
    key: "stop",
    value: function stop(id) {
      _helpers.ComponentEngine.stop({
        id: id,
        attribute: _constants.Selectors.DATA_TOOLTIP,
        thisArg: this,
        activeNodeKey: "_activeTooltip",
        cancelActiveFn: "_handleClose"
      });
    }
  }, {
    key: "_validate",
    value: function _validate(instance) {
      var instanceId = instance.getAttribute(_constants.Selectors.DATA_TOOLTIP);

      if (!instanceId) {
        (0, _helpers.log)(_constants.Messages.NO_ID_ERROR);
        return false;
      }

      var trigger = instance.querySelector(this._getTrigger(instanceId));
      var tooltip = instance.querySelector("#".concat(instanceId));

      if (!trigger) {
        (0, _helpers.log)(_constants.Messages.NO_TRIGGER_ERROR(instanceId));
        return false;
      }

      if (!tooltip) {
        (0, _helpers.log)(_constants.Messages.NO_TOOLTIP_ERROR(instanceId));
        return false;
      }

      trigger.setAttribute(_constants.Selectors.ARIA_DESCRIBEDBY, instanceId);
      tooltip.setAttribute(_constants.Selectors.ROLE, COMPONENT_ROLE);
      trigger.addEventListener(_constants.Events.MOUSEOVER, this._handleEvent);
      trigger.addEventListener(_constants.Events.FOCUS, this._handleEvent);
      return true;
    }
  }, {
    key: "_teardown",
    value: function _teardown(instance) {
      var id = instance.getAttribute(_constants.Selectors.DATA_TOOLTIP);
      var trigger = instance.querySelector(this._getTrigger(id));
      trigger.removeEventListener(_constants.Events.MOUSEOVER, this._handleEvent);
      trigger.removeEventListener(_constants.Events.FOCUS, this._handleEvent);
    }
  }, {
    key: "_handleEvent",
    value: function _handleEvent(event) {
      if (this._activeTooltip) this._handleClose();
      this._activeTrigger = event.target;

      var id = this._activeTrigger.getAttribute(_constants.Selectors.DATA_TARGET);

      this._activeTooltip = document.querySelector("[".concat(_constants.Selectors.DATA_TOOLTIP, "='").concat(id, "']"));
      this._activeTooltipBox = document.querySelector("#".concat(id));

      if (this._hasInlineClass()) {
        this._alignTooltip(_constants.CssProperties.HEIGHT);
      } else {
        this._alignTooltip(_constants.CssProperties.WIDTH);
      }

      this._setVisibleState();

      this._startCloseEvents();
    }
  }, {
    key: "_handleClose",
    value: function _handleClose() {
      if (this._activeTooltipBox) this._setHideState();

      this._startOpenEvents();

      this._resetProperties();
    }
  }, {
    key: "_resetProperties",
    value: function _resetProperties() {
      this._activeTooltip = null;
      this._activeTrigger = null;
      this._activeTooltipBox = null;
    }
  }, {
    key: "_setVisibleState",
    value: function _setVisibleState() {
      this._activeTooltipBox.setAttribute(_constants.Selectors.DATA_VISIBLE, "true");
    }
  }, {
    key: "_setHideState",
    value: function _setHideState() {
      this._activeTooltipBox.setAttribute(_constants.Selectors.DATA_VISIBLE, "false");
    }
  }, {
    key: "_startCloseEvents",
    value: function _startCloseEvents() {
      this._activeTrigger.removeEventListener(_constants.Events.MOUSEOVER, this._handleEvent);

      this._activeTrigger.removeEventListener(_constants.Events.FOCUS, this._handleEvent);

      this._activeTrigger.addEventListener(_constants.Events.MOUSEOUT, this._handleClose);

      this._activeTrigger.addEventListener(_constants.Events.BLUR, this._handleClose);

      document.addEventListener(_constants.Events.KEYDOWN, this._handleEscapeKeyPress);
      if (_helpers.isiOSMobile) document.body.classList.add(_constants.Selectors.OVERLAY_OPEN);
    }
  }, {
    key: "_handleEscapeKeyPress",
    value: function _handleEscapeKeyPress(event) {
      if (event.which === _constants.KeyCodes.ESCAPE) {
        this._handleClose();
      }
    }
  }, {
    key: "_startOpenEvents",
    value: function _startOpenEvents() {
      if (!this._activeTrigger) return;

      this._activeTrigger.removeEventListener(_constants.Events.MOUSEOUT, this._handleClose);

      this._activeTrigger.removeEventListener(_constants.Events.BLUR, this._handleClose);

      this._activeTrigger.addEventListener(_constants.Events.MOUSEOVER, this._handleEvent);

      this._activeTrigger.addEventListener(_constants.Events.FOCUS, this._handleEvent);

      document.removeEventListener(_constants.Events.KEYDOWN, this._handleEscapeKeyPress);
      if (_helpers.isiOSMobile) document.body.classList.remove(_constants.Selectors.OVERLAY_OPEN);
    }
  }, {
    key: "_alignTooltip",
    value: function _alignTooltip(property) {
      var triggerSize = this._getSize(this._activeTrigger, property);

      var tooltipSize = this._getSize(this._activeTooltipBox, property);

      var triggerIsBigger = triggerSize > tooltipSize;
      var offset = triggerIsBigger ? (triggerSize - tooltipSize) / 2 : (tooltipSize - triggerSize) / -2;

      if (property === _constants.CssProperties.HEIGHT) {
        this._activeTooltipBox.style[_constants.CssProperties.TOP] = "".concat(offset, "px");
      } else {
        this._activeTooltipBox.style[_constants.CssProperties.LEFT] = "".concat(offset, "px");
      }
    }
  }, {
    key: "_getTrigger",
    value: function _getTrigger(id) {
      return "[".concat(_constants.Selectors.DATA_TARGET, "=\"").concat(id, "\"]");
    }
  }, {
    key: "_getSize",
    value: function _getSize(element, property) {
      return Math.floor(element.getBoundingClientRect()[property]);
    }
  }, {
    key: "_hasInlineClass",
    value: function _hasInlineClass() {
      var classList = this._activeTooltipBox.classList;
      return classList.contains(_constants.Selectors.DROP_INLINE_START_CLASS) || classList.contains(_constants.Selectors.DROP_INLINE_END_CLASS);
    }
  }]);

  return Tooltip;
}();

exports.default = Tooltip;
//# sourceMappingURL=tooltip.js.map