function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

import { log, ComponentEngine, throttle } from "../helpers";
import { Selectors, CssProperties, CssValues, Events, Messages } from "./constants";

var Collapsible = function () {
  function Collapsible() {
    _classCallCheck(this, Collapsible);

    this._handleClick = throttle(this._handleClick.bind(this), 500);
    this._handleExpandTransition = this._handleExpandTransition.bind(this);
    this._validate = this._validate.bind(this);
    this._teardown = this._teardown.bind(this);
    this._components = [];
    this._activeCollapsible = {};
    this._activeTrigger = {};
    this._activeContent = {};
    this._activeId = "";
    this._nextAriaExpandedState = "";
    this._nextAriaHiddenState = "";
  }

  _createClass(Collapsible, [{
    key: "start",
    value: function start(id) {
      ComponentEngine.start({
        id: id,
        attribute: Selectors.DATA_COLLAPSIBLE,
        thisArg: this
      });
    }
  }, {
    key: "stop",
    value: function stop(id) {
      ComponentEngine.stop({
        id: id,
        attribute: Selectors.DATA_COLLAPSIBLE,
        thisArg: this,
        activeNodeKey: "_activeCollapsible"
      });
    }
  }, {
    key: "_validate",
    value: function _validate(instance) {
      var _this$_getCollapsible = this._getCollapsibleData(instance),
          trigger = _this$_getCollapsible.trigger,
          id = _this$_getCollapsible.id;

      if (!id) {
        log(Messages.NO_COLLAPSIBLE_ID_ERROR);
        return false;
      }

      if (!trigger) {
        log(Messages.NO_TRIGGER_ERROR(id));
        return false;
      }

      if (!trigger.id) {
        log(Messages.NO_TRIGGER_ID_ERROR(id));
        return false;
      }

      var contentId = "#".concat(id);
      var content = instance.querySelector(contentId);

      if (!content) {
        log(Messages.NO_CONTENT_ERROR(id));
        return false;
      }

      trigger.setAttribute(Selectors.ARIA_CONTROLS, id);
      content.setAttribute(Selectors.ARIA_LABELLEDBY, trigger.id);
      var contentIsVisible = instance.getAttribute(Selectors.DATA_VISIBLE) === "true";

      if (contentIsVisible) {
        instance.setAttribute(Selectors.DATA_VISIBLE, "true");
        trigger.setAttribute(Selectors.ARIA_EXPANDED, "true");
        content.setAttribute(Selectors.ARIA_HIDDEN, "false");
        content.style[CssProperties.HEIGHT] = "".concat(content.scrollHeight, "px");
        content.classList.add(Selectors.IS_VISIBLE_CLASS);
      } else {
        instance.setAttribute(Selectors.DATA_VISIBLE, "false");
        trigger.setAttribute(Selectors.ARIA_EXPANDED, "false");
        content.setAttribute(Selectors.ARIA_HIDDEN, "true");
      }

      requestAnimationFrame(function () {
        trigger.classList.add(Selectors.IS_READY_CLASS);
        content.classList.add(Selectors.IS_READY_CLASS);
      });
      trigger.addEventListener(Events.CLICK, this._handleClick);
      return true;
    }
  }, {
    key: "_teardown",
    value: function _teardown(instance) {
      var _this$_getCollapsible2 = this._getCollapsibleData(instance),
          trigger = _this$_getCollapsible2.trigger;

      trigger.removeEventListener(Events.CLICK, this._handleClick);
    }
  }, {
    key: "_handleClick",
    value: function _handleClick(event) {
      event.preventDefault();
      this._activeTrigger = event.target;

      this._setIds();

      this._setActiveCollapsible();

      this._setActiveContent();

      this._setNextVisibleState();

      this._toggleCollapsible();

      this._activeCollapsible = null;
    }
  }, {
    key: "_handleExpandTransition",
    value: function _handleExpandTransition() {
      this._activeContent.style[CssProperties.HEIGHT] = CssValues.AUTO;

      this._activeContent.removeEventListener(Events.TRANSITIONEND, this._handleExpandTransition);
    }
  }, {
    key: "_toggleCollapsible",
    value: function _toggleCollapsible() {
      this._activeCollapsible.setAttribute(Selectors.DATA_VISIBLE, this._nextAriaExpandedState);

      this._activeTrigger.setAttribute(Selectors.ARIA_EXPANDED, this._nextAriaExpandedState);

      this._activeContent.setAttribute(Selectors.ARIA_HIDDEN, this._nextAriaHiddenState);

      var fullHeightValue = "".concat(this._activeContent.scrollHeight, "px");

      if (this._activeCollapsible.getAttribute(Selectors.DATA_VISIBLE) === "false") {
        return this._collapsePanel(fullHeightValue);
      }

      this._expandPanel(fullHeightValue);
    }
  }, {
    key: "_expandPanel",
    value: function _expandPanel(height) {
      this._activeContent.style[CssProperties.HEIGHT] = height;

      this._activeContent.classList.add(Selectors.IS_VISIBLE_CLASS);

      this._activeContent.addEventListener(Events.TRANSITIONEND, this._handleExpandTransition);
    }
  }, {
    key: "_collapsePanel",
    value: function _collapsePanel(height) {
      var _this = this;

      return requestAnimationFrame(function () {
        _this._activeContent.style[CssProperties.HEIGHT] = height;
        requestAnimationFrame(function () {
          _this._activeContent.style[CssProperties.HEIGHT] = null;

          _this._activeContent.classList.remove(Selectors.IS_VISIBLE_CLASS);
        });
      });
    }
  }, {
    key: "_getCollapsibleData",
    value: function _getCollapsibleData(instance) {
      var id = instance.getAttribute(Selectors.DATA_COLLAPSIBLE);
      var trigger = instance.querySelector("[".concat(Selectors.DATA_TARGET, "='").concat(id, "']"));
      return {
        id: id,
        trigger: trigger
      };
    }
  }, {
    key: "_setActiveContent",
    value: function _setActiveContent() {
      this._activeContent = this._activeCollapsible.querySelector("#".concat(this._activeId));
    }
  }, {
    key: "_setNextVisibleState",
    value: function _setNextVisibleState() {
      var currentVisibleState = this._activeCollapsible.getAttribute(Selectors.DATA_VISIBLE);

      this._nextAriaExpandedState = currentVisibleState === "true" ? "false" : "true";
      this._nextAriaHiddenState = currentVisibleState === "false" ? "false" : "true";
    }
  }, {
    key: "_setIds",
    value: function _setIds() {
      this._activeId = this._activeTrigger.getAttribute(Selectors.DATA_TARGET);
    }
  }, {
    key: "_setActiveCollapsible",
    value: function _setActiveCollapsible() {
      this._activeCollapsible = document.querySelector("[".concat(Selectors.DATA_COLLAPSIBLE, "='").concat(this._activeId, "']"));
    }
  }]);

  return Collapsible;
}();

export { Collapsible as default };
//# sourceMappingURL=collapsible.js.map