function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

import { isiOSMobile, createFocusTrap, focusOnce, queryAll, log, ComponentEngine } from "../helpers";
import { KeyCodes, Selectors, Events, Messages } from "./constants";

var Dropdown = function () {
  function Dropdown() {
    _classCallCheck(this, Dropdown);

    this._handleClick = this._handleClick.bind(this);
    this._handleFirstTabClose = this._handleFirstTabClose.bind(this);
    this._handleLastTabClose = this._handleLastTabClose.bind(this);
    this._handleArrowKeyPress = this._handleArrowKeyPress.bind(this);
    this._handleClose = this._handleClose.bind(this);
    this._handleEscapeKeyPress = this._handleEscapeKeyPress.bind(this);
    this._handleOffMenuClick = this._handleOffMenuClick.bind(this);
    this._validate = this._validate.bind(this);
    this._teardown = this._teardown.bind(this);
    this._components = [];
    this._activeDropdownId = "";
    this._activeDropdownAttr = "";
    this._activeDropdownMenuId = "";
    this._activeDropdown = null;
    this._activeTrigger = null;
    this._activeDropdownMenu = null;
    this._firstDropdownAction = null;
    this._lastDropdownAction = null;
    this._focusTrap = null;
    this._activeDropdownActions = [];
    this._allowFocusReturn = true;
    this._dropdownContainerAttr = "[".concat(Selectors.DATA_DROPDOWN, "]");
    this._dropdownTargetAttr = "[".concat(Selectors.DATA_TARGET, "]");
    this._dropdownMenuClassName = ".".concat(Selectors.DROPDOWN_MENU_CLASS);
  }

  _createClass(Dropdown, [{
    key: "start",
    value: function start(id) {
      ComponentEngine.start({
        id: id,
        attribute: Selectors.DATA_DROPDOWN,
        thisArg: this
      });
    }
  }, {
    key: "stop",
    value: function stop(id) {
      ComponentEngine.stop({
        id: id,
        attribute: Selectors.DATA_DROPDOWN,
        thisArg: this,
        activeNodeKey: "_activeDropdown",
        cancelActiveFn: "_closeActiveDropdown"
      });
    }
  }, {
    key: "_validate",
    value: function _validate(instance) {
      var dropdownId = instance.getAttribute(Selectors.DATA_DROPDOWN);

      if (!dropdownId) {
        log(Messages.NO_DROPDOWN_ID_ERROR);
        return false;
      }

      var dropdownAttr = "[".concat(Selectors.DATA_DROPDOWN, "=\"").concat(dropdownId, "\"]");
      var dropdown = document.querySelector("[".concat(Selectors.DATA_DROPDOWN, "=\"").concat(dropdownId, "\"]"));
      var dropdownTrigger = dropdown.querySelector("[".concat(Selectors.DATA_TARGET, "]"));

      if (!dropdownTrigger.getAttribute(Selectors.DATA_PARENT)) {
        log(Messages.NO_PARENT_ERROR);
        return false;
      }

      var dropdownMenuId = dropdownTrigger.getAttribute(Selectors.DATA_TARGET);
      var dropdownMenu = dropdown.querySelector("#".concat(dropdownMenuId));

      if (!dropdownMenu) {
        log(Messages.NO_MENU_ERROR(dropdownAttr));
        return false;
      }

      dropdownMenu.setAttribute(Selectors.ARIA_LABELLEDBY, dropdownTrigger.id);
      dropdownTrigger.setAttribute(Selectors.ARIA_CONTROLS, dropdownMenuId);
      dropdownTrigger.setAttribute(Selectors.ARIA_HASPOPUP, "true");
      dropdownTrigger.setAttribute(Selectors.ARIA_EXPANDED, "false");
      var dropdownMenuListItems = dropdown.querySelectorAll("#".concat(dropdownMenuId, " > li"));

      if (!dropdownMenuListItems.length) {
        log(Messages.NO_DROPDOWN_ITEMS_ERROR(dropdownAttr));
        return false;
      }

      var dropdownMenuActions = this._getDropdownActions(dropdownAttr, "#".concat(dropdownMenuId));

      if (!dropdownMenuActions.length) {
        log(Messages.NO_DROPDOWN_ACTIONS_ERROR(dropdownAttr));
        return false;
      }

      dropdownMenuActions.forEach(function (trigger) {
        trigger.setAttribute(Selectors.TABINDEX, "-1");
      });
      dropdownTrigger.addEventListener(Events.CLICK, this._handleClick);
      dropdownTrigger.addEventListener(Events.KEYDOWN, this._handleArrowKeyPress);
      return true;
    }
  }, {
    key: "_teardown",
    value: function _teardown(instance) {
      var id = instance.getAttribute(Selectors.DATA_DROPDOWN);
      var trigger = instance.querySelector("[".concat(Selectors.DATA_PARENT, "='").concat(id, "']"));
      trigger.removeEventListener(Events.CLICK, this._handleClick);
      trigger.removeEventListener(Events.KEYDOWN, this._handleArrowKeyPress);
    }
  }, {
    key: "_handleClick",
    value: function _handleClick(event, key) {
      event.preventDefault();
      event.stopPropagation();

      this._closeOpenDropdowns(event);

      this._activeTrigger = event.target;

      this._setActiveDropdownId();

      this._setActiveDropdown();

      this._setActiveDropdownMenu();

      this._setVisibleState();

      this._startActiveDropdownEvents();

      if (key && key === KeyCodes.ARROW_UP) {
        this._lastDropdownAction.focus();
      } else {
        this._firstDropdownAction.focus();
      }

      if (isiOSMobile) document.body.classList.add(Selectors.OVERLAY_OPEN);
    }
  }, {
    key: "_handleClose",
    value: function _handleClose() {
      if (this._allowFocusReturn) this._handleReturnFocus();

      this._closeActiveDropdown();
    }
  }, {
    key: "_closeActiveDropdown",
    value: function _closeActiveDropdown() {
      var _this = this;

      if (isiOSMobile) document.body.classList.remove(Selectors.OVERLAY_OPEN);

      this._activeDropdown.setAttribute(Selectors.DATA_VISIBLE, "false");

      this._activeTrigger.setAttribute(Selectors.ARIA_EXPANDED, "false");

      this._activeTrigger.removeEventListener(Events.CLICK, this._handleClose);

      this._activeTrigger.addEventListener(Events.CLICK, this._handleClick);

      document.removeEventListener(Events.KEYDOWN, this._handleEscapeKeyPress);
      document.removeEventListener(Events.CLICK, this._handleOffMenuClick);

      this._firstDropdownAction.removeEventListener(Events.KEYDOWN, this._handleFirstTabClose);

      this._lastDropdownAction.removeEventListener(Events.KEYDOWN, this._handleLastTabClose);

      this._activeDropdownActions.forEach(function (action) {
        action.setAttribute(Selectors.TABINDEX, "-1");
        action.removeEventListener(Events.CLICK, _this._handleClose);
      });

      this._focusTrap.stop();

      this._focusTrap = null;

      this._resetProperties();
    }
  }, {
    key: "_resetProperties",
    value: function _resetProperties() {
      this._activeDropdownId = "";
      this._activeDropdownAttr = "";
      this._activeDropdownMenuId = "";
      this._activeDropdown = null;
      this._activeTrigger = null;
      this._activeDropdownMenu = null;
      this._firstDropdownAction = null;
      this._lastDropdownAction = null;
      this._focusTrap = null;
      this._activeDropdownActions = [];
      this._allowFocusReturn = true;
    }
  }, {
    key: "_setActiveDropdownId",
    value: function _setActiveDropdownId() {
      this._activeDropdownId = this._activeTrigger.getAttribute(Selectors.DATA_PARENT);
    }
  }, {
    key: "_startActiveDropdownEvents",
    value: function _startActiveDropdownEvents() {
      var _this2 = this;

      this._activeTrigger.removeEventListener(Events.CLICK, this._handleClick);

      this._activeTrigger.addEventListener(Events.CLICK, this._handleClose);

      document.addEventListener(Events.KEYDOWN, this._handleEscapeKeyPress);
      document.addEventListener(Events.CLICK, this._handleOffMenuClick);
      this._activeDropdownActions = this._getDropdownActions(this._activeDropdownAttr, "#".concat(this._activeDropdownMenuId));
      this._firstDropdownAction = this._activeDropdownActions[0];
      this._lastDropdownAction = this._activeDropdownActions[this._activeDropdownActions.length - 1];

      this._firstDropdownAction.addEventListener(Events.KEYDOWN, this._handleFirstTabClose);

      this._lastDropdownAction.addEventListener(Events.KEYDOWN, this._handleLastTabClose);

      this._activeDropdownActions.forEach(function (action) {
        action.setAttribute(Selectors.TABINDEX, "0");
        action.addEventListener(Events.CLICK, _this2._handleClose);
      });

      var containerSelector = "".concat(this._activeDropdownAttr, " > ").concat(this._dropdownMenuClassName);

      if (this._focusTrap) {
        this._focusTrap.stop();
      }

      this._focusTrap = createFocusTrap(containerSelector, {
        useArrows: true
      });

      this._focusTrap.start();
    }
  }, {
    key: "_setVisibleState",
    value: function _setVisibleState() {
      this._activeTrigger.setAttribute(Selectors.ARIA_EXPANDED, "true");

      this._activeDropdown.setAttribute(Selectors.DATA_VISIBLE, "true");
    }
  }, {
    key: "_setActiveDropdownMenu",
    value: function _setActiveDropdownMenu() {
      this._activeDropdownMenuId = this._activeTrigger.getAttribute(Selectors.DATA_TARGET);
      this._activeDropdownMenu = this._activeDropdown.querySelector("#".concat(this._activeDropdownMenuId));
    }
  }, {
    key: "_setActiveDropdown",
    value: function _setActiveDropdown() {
      this._activeDropdownAttr = "[".concat(Selectors.DATA_DROPDOWN, "=\"").concat(this._activeDropdownId, "\"]");
      this._activeDropdown = document.querySelector(this._activeDropdownAttr);
    }
  }, {
    key: "_closeOpenDropdowns",
    value: function _closeOpenDropdowns(event) {
      if (!this._activeTrigger) return;
      this._allowFocusReturn = false;

      this._handleClose(event);

      this._allowFocusReturn = true;
    }
  }, {
    key: "_handleFirstTabClose",
    value: function _handleFirstTabClose(event) {
      var shiftKey = event.which === KeyCodes.SHIFT || event.shiftKey;
      var tabKey = event.which === KeyCodes.TAB;

      if (shiftKey && tabKey) {
        this._handleClose(event);
      }
    }
  }, {
    key: "_handleLastTabClose",
    value: function _handleLastTabClose(event) {
      var shiftKey = event.which === KeyCodes.SHIFT || event.shiftKey;
      var tabKey = event.which === KeyCodes.TAB;

      if (tabKey && !shiftKey) {
        this._handleClose(event);
      }
    }
  }, {
    key: "_handleArrowKeyPress",
    value: function _handleArrowKeyPress(event) {
      if (event.which === KeyCodes.ARROW_UP || event.which === KeyCodes.ARROW_DOWN) {
        this._handleClick(event, event.which);
      }
    }
  }, {
    key: "_handleEscapeKeyPress",
    value: function _handleEscapeKeyPress(event) {
      if (event.which === KeyCodes.ESCAPE) {
        this._handleClose(event);
      }
    }
  }, {
    key: "_handleOffMenuClick",
    value: function _handleOffMenuClick(event) {
      if (event.target !== this._activeTrigger && event.target !== this._activeDropdownMenu) {
        this._handleClose(event);
      }
    }
  }, {
    key: "_handleReturnFocus",
    value: function _handleReturnFocus() {
      if (!this._activeTrigger) return;
      focusOnce(this._activeTrigger);
    }
  }, {
    key: "_getDropdownActions",
    value: function _getDropdownActions(dropdownAttr, menuId) {
      return queryAll("".concat(dropdownAttr, " > ").concat(menuId, " > li > a, ").concat(dropdownAttr, " > ").concat(menuId, " > li > button"));
    }
  }]);

  return Dropdown;
}();

export { Dropdown as default };
//# sourceMappingURL=dropdown.js.map