function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

import { isiOSMobile, getFocusableElements, createFocusTrap, focusOnce, log, queryAll, ComponentEngine } from "../helpers";
import { KeyCodes, Selectors, CssProperties, Events, Messages } from "./constants";
var COMPONENT_ROLE = "dialog";

var Modal = function () {
  function Modal() {
    _classCallCheck(this, Modal);

    this._handleClick = this._handleClick.bind(this);
    this._handleOpenTransition = this._handleOpenTransition.bind(this);
    this._handleCloseTransition = this._handleCloseTransition.bind(this);
    this._handleClose = this._handleClose.bind(this);
    this._handleOverlayClick = this._handleOverlayClick.bind(this);
    this._handleEscapeKeyPress = this._handleEscapeKeyPress.bind(this);
    this._validate = this._validate.bind(this);
    this._teardown = this._teardown.bind(this);
    this._components = [];
    this._activeModalTrigger = null;
    this._activeModalAttr = "";
    this._activeModal = null;
    this._activeModalContent = null;
    this._activeModalId = "";
    this._activeModalContentSelector = "";
    this._activeModalCloseTriggers = [];
    this._originalPagePadding = "";
    this._scrollbarOffset = null;
    this._focusTrap = null;
  }

  _createClass(Modal, [{
    key: "start",
    value: function start(id) {
      ComponentEngine.start({
        id: id,
        attribute: Selectors.DATA_MODAL,
        thisArg: this
      });
    }
  }, {
    key: "stop",
    value: function stop(id) {
      ComponentEngine.stop({
        id: id,
        attribute: Selectors.DATA_MODAL,
        thisArg: this,
        activeNodeKey: "_activeModal",
        cancelActiveFn: "_closeActiveModal"
      });
    }
  }, {
    key: "_validate",
    value: function _validate(instance) {
      var id = instance.getAttribute(Selectors.DATA_MODAL);

      if (!id) {
        log(Messages.NO_ID_ERROR);
        return false;
      }

      var modal = document.querySelector("[".concat(Selectors.DATA_MODAL, "='").concat(id, "']"));
      var modalContent = instance.querySelector("[".concat(Selectors.DATA_PARENT, "='").concat(id, "']"));

      if (!modalContent) {
        log(Messages.NO_MODAL_DIALOG_ERROR(id));
        return false;
      }

      modal.setAttribute(Selectors.ARIA_HIDDEN, "true");
      modal.setAttribute(Selectors.DATA_VISIBLE, "false");
      modalContent.setAttribute(Selectors.ARIA_MODAL, "true");
      modalContent.setAttribute(Selectors.ROLE, COMPONENT_ROLE);
      var trigger = document.querySelector("[".concat(Selectors.DATA_TARGET, "='").concat(id, "']"));

      if (!trigger) {
        log(Messages.NO_TRIGGER_ERROR(id));
        return false;
      }

      trigger.addEventListener(Events.CLICK, this._handleClick);
      return true;
    }
  }, {
    key: "_teardown",
    value: function _teardown(instance) {
      var id = instance.getAttribute(Selectors.DATA_MODAL);
      var trigger = document.querySelector("[".concat(Selectors.DATA_TARGET, "='").concat(id, "']"));
      trigger.removeEventListener(Events.CLICK, this._handleClick);
    }
  }, {
    key: "_handleClick",
    value: function _handleClick(event) {
      event.preventDefault();
      this._activeModalTrigger = event.target;

      this._setActiveId();

      this._setActiveModal();

      this._setActiveModalContent();

      this._setScrollbarOffset();

      this._setScrollStop();

      this._focusTrap = createFocusTrap(this._activeModalContentSelector);

      this._focusTrap.start();

      this._toggleVisibility(true);

      this._setFocusableChildren();

      this._setCloseTriggers();

      this._startEvents();
    }
  }, {
    key: "_handleClose",
    value: function _handleClose(event) {
      event.preventDefault();

      this._closeActiveModal();
    }
  }, {
    key: "_closeActiveModal",
    value: function _closeActiveModal() {
      var _this = this;

      this._toggleVisibility(false);

      this._focusTrigger();

      this._unsetScrollStop();

      this._unsetScrollbarOffset();

      document.removeEventListener(Events.KEYDOWN, this._handleEscapeKeyPress);
      document.removeEventListener(Events.CLICK, this._handleOverlayClick);

      this._activeModalCloseTriggers.forEach(function (trigger) {
        trigger.removeEventListener(Events.CLICK, _this._handleClose);
      });

      this._focusTrap.stop();

      this._resetProperties();
    }
  }, {
    key: "_resetProperties",
    value: function _resetProperties() {
      this._activeModal = null;
      this._activeModalTrigger = null;
      this._activeModalContent = null;
      this._activeModalId = "";
      this._activeModalContentSelector = "";
      this._activeModalCloseTriggers = [];
      this._originalPagePadding = "";
      this._scrollbarOffset = null;
      this._focusTrap = null;
    }
  }, {
    key: "_setFocusableChildren",
    value: function _setFocusableChildren() {
      var elements = getFocusableElements(this._activeModalContentSelector);
      if (!elements.length) return;
      elements.forEach(function (element) {
        return element.setAttribute(Selectors.TABINDEX, "0");
      });
    }
  }, {
    key: "_setCloseTriggers",
    value: function _setCloseTriggers() {
      this._activeModalCloseTriggers = queryAll("".concat(this._activeModalContentSelector, " [").concat(Selectors.DATA_CLOSE, "]"));
    }
  }, {
    key: "_setActiveId",
    value: function _setActiveId() {
      this._activeModalId = this._activeModalTrigger.getAttribute(Selectors.DATA_TARGET);
    }
  }, {
    key: "_setActiveModal",
    value: function _setActiveModal() {
      this._activeModalAttr = "[".concat(Selectors.DATA_MODAL, "='").concat(this._activeModalId, "']");
      this._activeModal = document.querySelector("[".concat(Selectors.DATA_MODAL, "='").concat(this._activeModalId, "']"));
    }
  }, {
    key: "_setActiveModalContent",
    value: function _setActiveModalContent() {
      this._activeModalContentSelector = "[".concat(Selectors.DATA_PARENT, "='").concat(this._activeModalId, "']");
      this._activeModalContent = this._activeModal.querySelector(this._activeModalContentSelector);
    }
  }, {
    key: "_handleOpenTransition",
    value: function _handleOpenTransition() {
      this._activeModal.removeEventListener(Events.TRANSITIONEND, this._handleOpenTransition);

      this._focusContent();

      this._activeModal.scrollTop = 0;
    }
  }, {
    key: "_handleCloseTransition",
    value: function _handleCloseTransition() {
      this._modalCache.style[CssProperties.PADDING_LEFT] = "";

      this._modalCache.removeEventListener(Events.TRANSITIONEND, this._handleCloseTransition);

      this._modalCache = null;
    }
  }, {
    key: "_toggleVisibility",
    value: function _toggleVisibility(isVisible) {
      this._activeModal.setAttribute(Selectors.ARIA_HIDDEN, isVisible ? "false" : "true");

      this._activeModal.setAttribute(Selectors.DATA_VISIBLE, isVisible ? "true" : "false");

      if (isVisible) {
        this._activeModal.classList.add(Selectors.IS_VISIBLE_CLASS);

        this._activeModal.addEventListener(Events.TRANSITIONEND, this._handleOpenTransition);
      } else {
        this._modalCache = this._activeModal;

        this._activeModal.classList.remove(Selectors.IS_VISIBLE_CLASS);

        this._activeModal.addEventListener(Events.TRANSITIONEND, this._handleCloseTransition);
      }
    }
  }, {
    key: "_startEvents",
    value: function _startEvents() {
      var _this2 = this;

      document.addEventListener(Events.KEYDOWN, this._handleEscapeKeyPress);
      document.addEventListener(Events.CLICK, this._handleOverlayClick);

      this._activeModalCloseTriggers.forEach(function (trigger) {
        trigger.addEventListener(Events.CLICK, _this2._handleClose);
      });
    }
  }, {
    key: "_getScrollbarOffset",
    value: function _getScrollbarOffset() {
      return window.innerWidth - document.body.getBoundingClientRect().right;
    }
  }, {
    key: "_setScrollbarOffset",
    value: function _setScrollbarOffset() {
      if (!this._scrollbarIsVisible()) return;
      this._scrollbarOffset = this._getScrollbarOffset();
      this._originalPagePadding = document.body.style[CssProperties.PADDING_RIGHT];
      document.body.style[CssProperties.PADDING_RIGHT] = "".concat(this._scrollbarOffset, "px");
    }
  }, {
    key: "_scrollbarIsVisible",
    value: function _scrollbarIsVisible() {
      if (typeof window.innerWidth === "number") {
        return window.innerWidth > document.body.getBoundingClientRect().right;
      }
    }
  }, {
    key: "_unsetScrollbarOffset",
    value: function _unsetScrollbarOffset() {
      if (!this._activeModal) return;
      var originalPaddingRight = this._originalPagePadding;
      this._activeModal.style[CssProperties.PADDING_LEFT] = "".concat(this._scrollbarOffset, "px");
      document.body.style[CssProperties.PADDING_RIGHT] = originalPaddingRight;
    }
  }, {
    key: "_handleOverlayClick",
    value: function _handleOverlayClick(event) {
      if (event.target === this._activeModal) {
        this._handleClose(event);
      }
    }
  }, {
    key: "_handleEscapeKeyPress",
    value: function _handleEscapeKeyPress(event) {
      if (event.which === KeyCodes.ESCAPE) {
        this._handleClose(event);
      }
    }
  }, {
    key: "_focusContent",
    value: function _focusContent() {
      focusOnce(this._activeModalContent);
    }
  }, {
    key: "_focusTrigger",
    value: function _focusTrigger() {
      focusOnce(this._activeModalTrigger);
    }
  }, {
    key: "_unsetScrollStop",
    value: function _unsetScrollStop() {
      if (isiOSMobile) document.body.classList.remove(Selectors.OVERLAY_OPEN);
      document.body.classList.remove(Selectors.NO_SCROLL_CLASS);
      document.documentElement.classList.remove(Selectors.NO_SCROLL_CLASS);
    }
  }, {
    key: "_setScrollStop",
    value: function _setScrollStop() {
      if (isiOSMobile) document.body.classList.add(Selectors.OVERLAY_OPEN);
      document.body.classList.add(Selectors.NO_SCROLL_CLASS);
      document.documentElement.classList.add(Selectors.NO_SCROLL_CLASS);
    }
  }]);

  return Modal;
}();

export { Modal as default };
//# sourceMappingURL=modal.js.map