function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

import { isiOSMobile, log, ComponentEngine } from "../helpers";
import { KeyCodes, Selectors, CssProperties, Events, Messages } from "./constants";
var COMPONENT_ROLE = "tooltip";

var Tooltip = function () {
  function Tooltip() {
    _classCallCheck(this, Tooltip);

    this._handleEvent = this._handleEvent.bind(this);
    this._handleClose = this._handleClose.bind(this);
    this._handleEscapeKeyPress = this._handleEscapeKeyPress.bind(this);
    this._validate = this._validate.bind(this);
    this._teardown = this._teardown.bind(this);
    this._components = [];
    this._activeTrigger = null;
    this._activeTooltipBox = null;
  }

  _createClass(Tooltip, [{
    key: "start",
    value: function start(id) {
      ComponentEngine.start({
        id: id,
        attribute: Selectors.DATA_TOOLTIP,
        thisArg: this
      });
    }
  }, {
    key: "stop",
    value: function stop(id) {
      ComponentEngine.stop({
        id: id,
        attribute: Selectors.DATA_TOOLTIP,
        thisArg: this,
        activeNodeKey: "_activeTooltip",
        cancelActiveFn: "_handleClose"
      });
    }
  }, {
    key: "_validate",
    value: function _validate(instance) {
      var instanceId = instance.getAttribute(Selectors.DATA_TOOLTIP);

      if (!instanceId) {
        log(Messages.NO_ID_ERROR);
        return false;
      }

      var trigger = instance.querySelector(this._getTrigger(instanceId));
      var tooltip = instance.querySelector("#".concat(instanceId));

      if (!trigger) {
        log(Messages.NO_TRIGGER_ERROR(instanceId));
        return false;
      }

      if (!tooltip) {
        log(Messages.NO_TOOLTIP_ERROR(instanceId));
        return false;
      }

      trigger.setAttribute(Selectors.ARIA_DESCRIBEDBY, instanceId);
      tooltip.setAttribute(Selectors.ROLE, COMPONENT_ROLE);
      trigger.addEventListener(Events.MOUSEOVER, this._handleEvent);
      trigger.addEventListener(Events.FOCUS, this._handleEvent);
      return true;
    }
  }, {
    key: "_teardown",
    value: function _teardown(instance) {
      var id = instance.getAttribute(Selectors.DATA_TOOLTIP);
      var trigger = instance.querySelector(this._getTrigger(id));
      trigger.removeEventListener(Events.MOUSEOVER, this._handleEvent);
      trigger.removeEventListener(Events.FOCUS, this._handleEvent);
    }
  }, {
    key: "_handleEvent",
    value: function _handleEvent(event) {
      if (this._activeTooltip) this._handleClose();
      this._activeTrigger = event.target;

      var id = this._activeTrigger.getAttribute(Selectors.DATA_TARGET);

      this._activeTooltip = document.querySelector("[".concat(Selectors.DATA_TOOLTIP, "='").concat(id, "']"));
      this._activeTooltipBox = document.querySelector("#".concat(id));

      if (this._hasInlineClass()) {
        this._alignTooltip(CssProperties.HEIGHT);
      } else {
        this._alignTooltip(CssProperties.WIDTH);
      }

      this._setVisibleState();

      this._startCloseEvents();
    }
  }, {
    key: "_handleClose",
    value: function _handleClose() {
      if (this._activeTooltipBox) this._setHideState();

      this._startOpenEvents();

      this._resetProperties();
    }
  }, {
    key: "_resetProperties",
    value: function _resetProperties() {
      this._activeTooltip = null;
      this._activeTrigger = null;
      this._activeTooltipBox = null;
    }
  }, {
    key: "_setVisibleState",
    value: function _setVisibleState() {
      this._activeTooltipBox.setAttribute(Selectors.DATA_VISIBLE, "true");
    }
  }, {
    key: "_setHideState",
    value: function _setHideState() {
      this._activeTooltipBox.setAttribute(Selectors.DATA_VISIBLE, "false");
    }
  }, {
    key: "_startCloseEvents",
    value: function _startCloseEvents() {
      this._activeTrigger.removeEventListener(Events.MOUSEOVER, this._handleEvent);

      this._activeTrigger.removeEventListener(Events.FOCUS, this._handleEvent);

      this._activeTrigger.addEventListener(Events.MOUSEOUT, this._handleClose);

      this._activeTrigger.addEventListener(Events.BLUR, this._handleClose);

      document.addEventListener(Events.KEYDOWN, this._handleEscapeKeyPress);
      if (isiOSMobile) document.body.classList.add(Selectors.OVERLAY_OPEN);
    }
  }, {
    key: "_handleEscapeKeyPress",
    value: function _handleEscapeKeyPress(event) {
      if (event.which === KeyCodes.ESCAPE) {
        this._handleClose();
      }
    }
  }, {
    key: "_startOpenEvents",
    value: function _startOpenEvents() {
      if (!this._activeTrigger) return;

      this._activeTrigger.removeEventListener(Events.MOUSEOUT, this._handleClose);

      this._activeTrigger.removeEventListener(Events.BLUR, this._handleClose);

      this._activeTrigger.addEventListener(Events.MOUSEOVER, this._handleEvent);

      this._activeTrigger.addEventListener(Events.FOCUS, this._handleEvent);

      document.removeEventListener(Events.KEYDOWN, this._handleEscapeKeyPress);
      if (isiOSMobile) document.body.classList.remove(Selectors.OVERLAY_OPEN);
    }
  }, {
    key: "_alignTooltip",
    value: function _alignTooltip(property) {
      var triggerSize = this._getSize(this._activeTrigger, property);

      var tooltipSize = this._getSize(this._activeTooltipBox, property);

      var triggerIsBigger = triggerSize > tooltipSize;
      var offset = triggerIsBigger ? (triggerSize - tooltipSize) / 2 : (tooltipSize - triggerSize) / -2;

      if (property === CssProperties.HEIGHT) {
        this._activeTooltipBox.style[CssProperties.TOP] = "".concat(offset, "px");
      } else {
        this._activeTooltipBox.style[CssProperties.LEFT] = "".concat(offset, "px");
      }
    }
  }, {
    key: "_getTrigger",
    value: function _getTrigger(id) {
      return "[".concat(Selectors.DATA_TARGET, "=\"").concat(id, "\"]");
    }
  }, {
    key: "_getSize",
    value: function _getSize(element, property) {
      return Math.floor(element.getBoundingClientRect()[property]);
    }
  }, {
    key: "_hasInlineClass",
    value: function _hasInlineClass() {
      var classList = this._activeTooltipBox.classList;
      return classList.contains(Selectors.DROP_INLINE_START_CLASS) || classList.contains(Selectors.DROP_INLINE_END_CLASS);
    }
  }]);

  return Tooltip;
}();

export { Tooltip as default };
//# sourceMappingURL=tooltip.js.map